#!/usr/bin/env fades

import argparse
import os
import sys

import requests  # fades
import httplib2  # fades
from apiclient import discovery  # fades google-api-python-client-py3
from oauth2client import client, tools  # fades
from oauth2client.file import Storage

# hack to fix 'apiclient' for modern Python
from collections.abc import Callable
import collections
collections.Callable = Callable

SCOPE = 'https://www.googleapis.com/auth/drive'
APPLICATION_NAME = 'Auto photo downloader'


credentials_file = "/home/facundo/.credentials/tablet_foto_autoupload.json"
client_secret_file = "/home/facundo/.credentials/tablet_client_secret.json"


def get_credentials(auth_flags):
    """Gets valid user credentials from storage.

    If nothing has been stored, or if the stored credentials are invalid,
    the OAuth2 flow is completed to obtain the new credentials.

    Returns:
        Credentials, the obtained credential.
    """
    store = Storage(credentials_file)
    credentials = store.get()
    if not credentials or credentials.invalid:
        flow = client.flow_from_clientsecrets(client_secret_file, SCOPE)
        flow.user_agent = APPLICATION_NAME
        credentials = tools.run_flow(flow, store, auth_flags)
        print("Storing credentials to", credentials_file)
    return credentials


auth_flags = argparse.ArgumentParser(parents=[tools.argparser]).parse_args()
credentials = get_credentials(auth_flags)
http = credentials.authorize(httplib2.Http())
service = discovery.build('drive', 'v3', http=http)

# https://drive.google.com/file/d/15shUxk0Pz7H-OqTPGHiuvk3Pbdf11XYn/view?usp=share_link
file_id = "15shUxk0Pz7H-OqTPGHiuvk3Pbdf11XYn"

# get the filename and file size
obj = service.files().get(fileId=file_id, fields="name,size").execute()
filename = obj.get("name", "sampleName")
size = obj.get("size", None)
if not size:
    sys.exit("No file size.")
else:
    size = int(size)
print("File found!", repr(filename), size)

# Check existing file.
file_path = os.path.join("./", filename)
params = {}
if os.path.exists(file_path):
    params["start_byte"] = os.path.getsize(file_path)
    params["mode"] = "ab"
    params["download"] = "As resume"
else:
    params["start_byte"] = 0
    params["mode"] = "wb"
    params["download"] = "As a new file"
if params["start_byte"] == size:
    sys.exit("The download of this file has already been finished.")

# Download process
print("Parameters for download:", params)
headers = {
    "Authorization": f"Bearer {credentials.access_token}",
    "Range": f'bytes={params["start_byte"]}-',
}
url = f"https://www.googleapis.com/drive/v3/files/{file_id}?alt=media"
tot = params["start_byte"]
with requests.get(url, headers=headers, stream=True) as r:
    r.raise_for_status()
    with open(file_path, params["mode"]) as f:
        for chunk in r.iter_content(chunk_size=1024 * 1024):
            tot += len(chunk)
            f.write(chunk)
            print(tot / (1024 * 1024))
